function sDot = quadEOM_readonly(t, s, F, M, param)
% QUADEOM_READONLY Solve quadrotor equation of motion
%   quadEOM_readonly calculate the derivative of the state vector
%
% INPUTS:
% t      - 1 x 1, time
% s      - 13 x 1, state vector = [x, y, z, xd, yd, zd, qw, qx, qy, qz, p, q, r]
% F      - 1 x 1, thrust output from controller (only used in simulation)
% M      - 3 x 1, moments output from controller (only used in simulation)
% param - struct, output from phyfile() and whatever parameters you want to pass in
%
% OUTPUTS:
% sDot   - 13 x 1, derivative of state vector s
%
% NOTE: You should not modify this function
% See Also: quadEOM_readonly, crazyflie

%************ EQUATIONS OF MOTION ************************
% Limit the force and moments due to actuator limits
A = [0.25,                      0, -0.5/param.arm_length;
     0.25,  0.5/param.arm_length,                      0;
     0.25,                      0,  0.5/param.arm_length;
     0.25, -0.5/param.arm_length,                      0];

prop_thrusts = A*[F;M(1:2)]; % Not using moment about Z-axis for limits
prop_thrusts_clamped = max(min(prop_thrusts, param.maxF/4), param.minF/4);

B = [                 1,                 1,                 1,                  1;
                      0, param.arm_length,                 0, -param.arm_length;
     -param.arm_length,                 0, param.arm_length,                 0];
F = B(1,:)*prop_thrusts_clamped;
M = [B(2:3,:)*prop_thrusts_clamped; M(3)];

% Assign states
x = s(1);
y = s(2);
z = s(3);
xDot = s(4);
yDot = s(5);
zDot = s(6);
qW = s(7);
qX = s(8);
qY = s(9);
qZ = s(10);
p = s(11);
q = s(12);
r = s(13);

%Quaternion: Orientation (attitude) of the drone in the world frame
quat = [qW; qX; qY; qZ]; %angle, e_x, e_y, e_z
bRw = QuatToRot(quat);
wRb = bRw';

% Acceleration
accel = 1 / param.mass * (wRb * [0; 0; F] - [0; 0; param.mass * param.grav]);

% Angular velocity
K_quat = 2; %this enforces the magnitude 1 constraint for the quaternion
quaterror = 1 - (qW^2 + qX^2 + qY^2 + qZ^2);
qDot = -1/2*[0, -p, -q, -r;...
             p,  0, -r,  q;...
             q,  r,  0, -p;...
             r, -q,  p,  0] * quat + K_quat*quaterror * quat;

% Angular acceleration
omega = [p;q;r];
pqrDot   = param.invI * (M - cross(omega, param.I*omega));

% Assemble sDot
sDot = zeros(13,1);
sDot(1)  = xDot;
sDot(2)  = yDot;
sDot(3)  = zDot;
sDot(4)  = accel(1);
sDot(5)  = accel(2);
sDot(6)  = accel(3);
sDot(7)  = qDot(1);
sDot(8)  = qDot(2);
sDot(9)  = qDot(3);
sDot(10) = qDot(4);
sDot(11) = pqrDot(1);
sDot(12) = pqrDot(2);
sDot(13) = pqrDot(3);

end